@@ -1,4 +1,4 @@
-import os
+"""classicML的优化器."""
 from time import time
 
 import numpy as np
@@ -72,7 +72,8 @@ class Optimizer(object):
         self._progressbar = None
 
     def __call__(self, x, y, epochs, parameters, *args, **kwargs):
-        """
+        """函数实现.
+
         Arguments:
             x: numpy.ndarray, array-like, 特征数据.
             y: numpy.ndarray, array-like, 标签.
@@ -102,255 +103,14 @@ class Optimizer(object):
         raise NotImplementedError
 
 
-class GradientDescent(Optimizer):
-    """梯度下降优化器.
-    """
-    def __init__(self, name='gradient_descent', learning_rate=1e-2):
-        """
-        Arguments:
-            name: str, default=None,
-                优化器的名称.
-            learning_rate: float, default=1e-2,
-                优化器的学习率.
-        """
-        super(GradientDescent, self).__init__(name=name)
-        self.learning_rate = learning_rate
-
-    def run(self, x, y, epochs, beta, *args, **kwargs):
-        """运行优化器优化参数.
-
-        Arguments:
-            x: numpy.ndarray, array-like, 特征数据.
-            y: numpy.ndarray, array-like, 标签.
-            epochs: int, default=1, 训练的轮数.
-            beta: numpy.ndarray, 模型的参数矩阵.
-            args:
-                verbose: bool, 显示日志信息.
-                loss: str, classicML.losses.Loss 实例,
-                    模型使用的损失函数.
-                metric: str, classicML.metrics.Metric 实例,
-                    模型使用的评估函数.
-        Returns:
-            模型的参数矩阵.
-        """
-        verbose, loss, metric, callbacks = _get_optimizer_parameters(args, kwargs)
-        if verbose is True:
-            # 实例化进度条
-            self._progressbar = ProgressBar(epochs, loss, metric)
-
-        for epoch in range(1, epochs+1):
-            current = time()
-            # 前向传播
-            y_pred, x_hat = self.forward(x, beta)
-            # 反向传播
-            grad = self.backward(y_pred, y, x_hat)
-            # 更新参数
-            beta = self._update_parameters(beta, grad)
-            # 以下的操作是为了更好可视化的, 不参与实际优化
-            if self._progressbar or callbacks is not None:
-                if loss.name is 'log_likelihood':
-                    loss_value = loss(y, beta, x_hat)
-                else:
-                    loss_value = loss(y_pred, y)
-                metric_value = metric(y_pred, y)
-
-                # 记录callbacks
-                if callbacks:
-                    _record_callbacks(callbacks, loss_value, metric_value)
-                # 进度条显示
-                if self._progressbar:
-                    self._progressbar(epoch, current, loss_value, metric_value)
-
-        return beta
-
-    def _update_parameters(self, beta, grad, *args, **kwargs):
-        """更新模型的参数.
-
-        Arguments:
-            beta: numpy.ndarray, 模型的参数矩阵.
-            grad: numpy.ndarray, 优化器的实时梯度矩阵.
-        """
-        beta -= self.learning_rate * grad
-
-        return beta
-
-    @staticmethod
-    def forward(x, parameters):
-        """优化器前向传播.
-
-        Arguments:
-            x: numpy.ndarray, array-like, 特征数据.
-            parameters: numpy.ndarray, 模型的参数矩阵.
-
-        Returns:
-            预测的标签(概率形式)和参数(x;1)矩阵.
-        """
-        number_of_sample = x.shape[0]
-        parameters = parameters.reshape(-1, 1)
-
-        x_hat = np.c_[x, np.ones((number_of_sample, 1))]
-        cache = np.matmul(x_hat, parameters)
-        y_pred = sigmoid(cache)
-
-        return y_pred, x_hat
-
-    @staticmethod
-    def backward(y_pred, y_true, x_hat):
-        """优化器反向传播.
-
-        Arguments:
-            y_pred: numpy.ndarray, 预测的标签(概率形式).
-            y_true: numpy.ndarray, 真实的标签.
-            x_hat: numpy.ndarray, 属性的参数矩阵.
-
-        Returns:
-            优化器的实时梯度矩阵.
-        """
-        y_true = y_true.reshape(-1, 1)
-        error = y_true - y_pred
-        grad = np.sum((-x_hat * error), axis=0)
-        grad = grad.reshape(-1, 1)
-
-        return grad
-
-
-class NewtonMethod(Optimizer):
-    """牛顿法优化器.
-    """
-    def __init__(self, name='newton_method'):
-        """
-        Arguments:
-            name: str, default=None,
-                优化器的名称.
-        """
-        super(NewtonMethod, self).__init__(name=name)
-
-    def run(self, x, y, epochs, beta, *args, **kwargs):
-        """运行优化器优化参数.
-
-        Arguments:
-            x: numpy.ndarray, array-like, 特征数据.
-            y: numpy.ndarray, array-like, 标签.
-            epochs: int, default=1, 训练的轮数.
-            beta: numpy.ndarray, 模型的参数矩阵.
-            args:
-                verbose: bool, 显示日志信息.
-                loss: str, classicML.losses.Loss 实例,
-                    模型使用的损失函数.
-                metric: str, classicML.metrics.Metric 实例,
-                    模型使用的评估函数.
-        Returns:
-            模型的参数矩阵.
-        """
-        verbose, loss, metric, callbacks = _get_optimizer_parameters(args, kwargs)
-        if verbose:
-            # 实例化进度条
-            self._progressbar = ProgressBar(epochs, loss, metric)
-
-        for epoch in range(1, epochs + 1):
-            current = time()
-            # 前向传播
-            y_pred, x_hat = self.forward(x, beta)
-            # 反向传播
-            grad = self.backward(y_pred, y, x_hat)
-            # 求解海森矩阵
-            hessian = self._get_hessian_matrix(y_pred, x_hat)
-            # 更新参数
-            beta = self._update_parameters(beta, grad, hessian)
-            # 以下的操作是为了更好可视化的, 不参与实际优化
-            if self._progressbar or callbacks is not None:
-                if loss.name is 'log_likelihood':
-                    loss_value = loss(y, beta, x_hat)
-                else:
-                    loss_value = loss(y_pred, y)
-                metric_value = metric(y_pred, y)
-
-                # 进度条显示
-                if self._progressbar:
-                    self._progressbar(epoch, current, loss_value, metric_value)
-                # 记录callbacks
-                if callbacks:
-                    _record_callbacks(callbacks, loss_value, metric_value)
-
-        return beta
-
-    def _update_parameters(self, beta, grad, *args, **kwargs):
-        """更新模型的参数.
-
-        Arguments:
-            beta: numpy.ndarray, 模型的参数矩阵.
-            grad: numpy.ndarray, 优化器的实时梯度矩阵.
-            args:
-                hessian: numpy.ndarray, 海森矩阵.
-        """
-        hessian = args[0]
-        beta -= np.matmul(np.linalg.inv(hessian), grad)
-
-        return beta
-
-    @staticmethod
-    def forward(x, parameters):
-        """优化器前向传播.
-
-        Arguments:
-            x: numpy.ndarray, array-like, 特征数据.
-            parameters: numpy.ndarray, 模型的参数矩阵.
-
-        Returns:
-            预测的标签(概率形式)和参数(x;1)矩阵.
-        """
-        number_of_sample = x.shape[0]
-        parameters = parameters.reshape(-1, 1)
-
-        x_hat = np.c_[x, np.ones((number_of_sample, 1))]
-        cache = np.matmul(x_hat, parameters)
-        y_pred = sigmoid(cache)
-
-        return y_pred, x_hat
-
-    @staticmethod
-    def backward(y_pred, y_true, x_hat):
-        """优化器反向传播.
-
-        Arguments:
-            y_pred: numpy.ndarray, 预测的标签(概率形式).
-            y_true: numpy.ndarray, 真实的标签.
-            x_hat: numpy.ndarray, 属性的参数矩阵.
-
-        Returns:
-            优化器的实时梯度矩阵.
-        """
-        y_true = y_true.reshape(-1, 1)
-        error = y_true - y_pred
-        grad = np.sum((-x_hat * error), axis=0)
-        grad = grad.reshape(-1, 1)
-
-        return grad
-
-    @staticmethod
-    def _get_hessian_matrix(y_pred, x_hat):
-        """计算海森矩阵(二阶导数).
-
-        Arguments:
-            y_pred: numpy.ndarray, 预测的标签(概率形式).
-            x_hat: numpy.ndarray, 属性的参数矩阵.
-
-        Returns:
-            海森矩阵.
-        """
-        P = np.eye(x_hat.shape[0]) * y_pred * (1 - y_pred)
-        H = np.matmul(np.matmul(x_hat.T, P), x_hat)
-
-        return H
-
-
 class StochasticGradientDescent(Optimizer):
     """随机梯度下降优化器.
 
     Notes:
-        - 如果想固定随机种子实现, 实现复现的话,
+        - 如果想固定随机种子, 实现复现的话,
           请在模型实例化的时候将随机种子置为一个常整数.
     """
+
     def __init__(self, name='stochastic_gradient_descent', learning_rate=1e-2):
         """
         Arguments:
@@ -565,6 +325,7 @@ class Adam(StochasticGradientDescent):
         - 超参数epsilon按照其他的机器学习框架设置的1e-7,
           非原论文的1e-8.
     """
+
     def __init__(self,
                  name='adam',
                  learning_rate=1e-3,
@@ -606,6 +367,7 @@ class Adam(StochasticGradientDescent):
                     模型使用的评估函数.
                 callbacks: list,
                     模型训练过程的中间数据记录器.
+
         Returns:
             模型的参数矩阵.
         """
@@ -686,7 +448,7 @@ class Adam(StochasticGradientDescent):
             parameters['b' + str(i)] -= self.learning_rate * m_hat_b / (np.sqrt(v_hat_b) + self.epsilon)
 
         return parameters, m, v
-    
+
     @staticmethod
     def _initializer(parameters):
         """初始化一阶和二阶向量, 算法一实现全初始化为零.
@@ -712,6 +474,249 @@ class Adam(StochasticGradientDescent):
         return moment_vector_1, moment_vector_2
 
 
+class GradientDescent(Optimizer):
+    """梯度下降优化器.
+    """
+    def __init__(self, name='gradient_descent', learning_rate=1e-2):
+        """
+        Arguments:
+            name: str, default=None,
+                优化器的名称.
+            learning_rate: float, default=1e-2,
+                优化器的学习率.
+        """
+        super(GradientDescent, self).__init__(name=name)
+        self.learning_rate = learning_rate
+
+    def run(self, x, y, epochs, beta, *args, **kwargs):
+        """运行优化器优化参数.
+
+        Arguments:
+            x: numpy.ndarray, array-like, 特征数据.
+            y: numpy.ndarray, array-like, 标签.
+            epochs: int, default=1, 训练的轮数.
+            beta: numpy.ndarray, 模型的参数矩阵.
+            args:
+                verbose: bool, 显示日志信息.
+                loss: str, classicML.losses.Loss 实例,
+                    模型使用的损失函数.
+                metric: str, classicML.metrics.Metric 实例,
+                    模型使用的评估函数.
+        Returns:
+            模型的参数矩阵.
+        """
+        verbose, loss, metric, callbacks = _get_optimizer_parameters(args, kwargs)
+        if verbose is True:
+            # 实例化进度条
+            self._progressbar = ProgressBar(epochs, loss, metric)
+
+        for epoch in range(1, epochs+1):
+            current = time()
+            # 前向传播
+            y_pred, x_hat = self.forward(x, beta)
+            # 反向传播
+            grad = self.backward(y_pred, y, x_hat)
+            # 更新参数
+            beta = self._update_parameters(beta, grad)
+            # 以下的操作是为了更好可视化的, 不参与实际优化
+            if self._progressbar or callbacks is not None:
+                if loss.name == 'log_likelihood':
+                    loss_value = loss(y, beta, x_hat)
+                else:
+                    loss_value = loss(y_pred, y)
+                metric_value = metric(y_pred, y)
+
+                # 记录callbacks
+                if callbacks:
+                    _record_callbacks(callbacks, loss_value, metric_value)
+                # 进度条显示
+                if self._progressbar:
+                    self._progressbar(epoch, current, loss_value, metric_value)
+
+        return beta
+
+    def _update_parameters(self, beta, grad, *args, **kwargs):
+        """更新模型的参数.
+
+        Arguments:
+            beta: numpy.ndarray, 模型的参数矩阵.
+            grad: numpy.ndarray, 优化器的实时梯度矩阵.
+        """
+        beta -= self.learning_rate * grad
+
+        return beta
+
+    @staticmethod
+    def forward(x, parameters):
+        """优化器前向传播.
+
+        Arguments:
+            x: numpy.ndarray, array-like, 特征数据.
+            parameters: numpy.ndarray, 模型的参数矩阵.
+
+        Returns:
+            预测的标签(概率形式)和参数(x;1)矩阵.
+        """
+        number_of_sample = x.shape[0]
+        parameters = parameters.reshape(-1, 1)
+
+        x_hat = np.c_[x, np.ones((number_of_sample, 1))]
+        cache = np.matmul(x_hat, parameters)
+        y_pred = sigmoid(cache)
+
+        return y_pred, x_hat
+
+    @staticmethod
+    def backward(y_pred, y_true, x_hat):
+        """优化器反向传播.
+
+        Arguments:
+            y_pred: numpy.ndarray, 预测的标签(概率形式).
+            y_true: numpy.ndarray, 真实的标签.
+            x_hat: numpy.ndarray, 属性的参数矩阵.
+
+        Returns:
+            优化器的实时梯度矩阵.
+        """
+        y_true = y_true.reshape(-1, 1)
+        error = y_true - y_pred
+        grad = np.sum((-x_hat * error), axis=0)
+        grad = grad.reshape(-1, 1)
+
+        return grad
+
+
+class NewtonMethod(Optimizer):
+    """牛顿法优化器.
+    """
+    def __init__(self, name='newton_method'):
+        """
+        Arguments:
+            name: str, default=None,
+                优化器的名称.
+        """
+        super(NewtonMethod, self).__init__(name=name)
+
+    def run(self, x, y, epochs, beta, *args, **kwargs):
+        """运行优化器优化参数.
+
+        Arguments:
+            x: numpy.ndarray, array-like, 特征数据.
+            y: numpy.ndarray, array-like, 标签.
+            epochs: int, default=1, 训练的轮数.
+            beta: numpy.ndarray, 模型的参数矩阵.
+            args:
+                verbose: bool, 显示日志信息.
+                loss: str, classicML.losses.Loss 实例,
+                    模型使用的损失函数.
+                metric: str, classicML.metrics.Metric 实例,
+                    模型使用的评估函数.
+
+        Returns:
+            模型的参数矩阵.
+        """
+        verbose, loss, metric, callbacks = _get_optimizer_parameters(args, kwargs)
+        if verbose:
+            # 实例化进度条
+            self._progressbar = ProgressBar(epochs, loss, metric)
+
+        for epoch in range(1, epochs + 1):
+            current = time()
+            # 前向传播
+            y_pred, x_hat = self.forward(x, beta)
+            # 反向传播
+            grad = self.backward(y_pred, y, x_hat)
+            # 求解海森矩阵
+            hessian = self._get_hessian_matrix(y_pred, x_hat)
+            # 更新参数
+            beta = self._update_parameters(beta, grad, hessian)
+            # 以下的操作是为了更好可视化的, 不参与实际优化
+            if self._progressbar or callbacks is not None:
+                if loss.name == 'log_likelihood':
+                    loss_value = loss(y, beta, x_hat)
+                else:
+                    loss_value = loss(y_pred, y)
+                metric_value = metric(y_pred, y)
+
+                # 进度条显示
+                if self._progressbar:
+                    self._progressbar(epoch, current, loss_value, metric_value)
+                # 记录callbacks
+                if callbacks:
+                    _record_callbacks(callbacks, loss_value, metric_value)
+
+        return beta
+
+    def _update_parameters(self, beta, grad, *args, **kwargs):
+        """更新模型的参数.
+
+        Arguments:
+            beta: numpy.ndarray, 模型的参数矩阵.
+            grad: numpy.ndarray, 优化器的实时梯度矩阵.
+            args:
+                hessian: numpy.ndarray, 海森矩阵.
+        """
+        hessian = args[0]
+        beta -= np.matmul(np.linalg.inv(hessian), grad)
+
+        return beta
+
+    @staticmethod
+    def forward(x, parameters):
+        """优化器前向传播.
+
+        Arguments:
+            x: numpy.ndarray, array-like, 特征数据.
+            parameters: numpy.ndarray, 模型的参数矩阵.
+
+        Returns:
+            预测的标签(概率形式)和参数(x;1)矩阵.
+        """
+        number_of_sample = x.shape[0]
+        parameters = parameters.reshape(-1, 1)
+
+        x_hat = np.c_[x, np.ones((number_of_sample, 1))]
+        cache = np.matmul(x_hat, parameters)
+        y_pred = sigmoid(cache)
+
+        return y_pred, x_hat
+
+    @staticmethod
+    def backward(y_pred, y_true, x_hat):
+        """优化器反向传播.
+
+        Arguments:
+            y_pred: numpy.ndarray, 预测的标签(概率形式).
+            y_true: numpy.ndarray, 真实的标签.
+            x_hat: numpy.ndarray, 属性的参数矩阵.
+
+        Returns:
+            优化器的实时梯度矩阵.
+        """
+        y_true = y_true.reshape(-1, 1)
+        error = y_true - y_pred
+        grad = np.sum((-x_hat * error), axis=0)
+        grad = grad.reshape(-1, 1)
+
+        return grad
+
+    @staticmethod
+    def _get_hessian_matrix(y_pred, x_hat):
+        """计算海森矩阵(二阶导数).
+
+        Arguments:
+            y_pred: numpy.ndarray, 预测的标签(概率形式).
+            x_hat: numpy.ndarray, 属性的参数矩阵.
+
+        Returns:
+            海森矩阵.
+        """
+        P = np.eye(x_hat.shape[0]) * y_pred * (1 - y_pred)
+        H = np.matmul(np.matmul(x_hat.T, P), x_hat)
+
+        return H
+
+
 class RadialBasisFunctionOptimizer(Optimizer):
     """径向基函数优化器.
     """
@@ -896,7 +901,7 @@ class SequentialMinimalOptimization(Optimizer):
             args:
                 C: float,
                     软间隔正则化系数.
-                kernel: str, classicML.kernel.Kernels 实例,
+                kernel: str, classicML.kernels.Kernel 实例,
                     分类器使用的核函数.
                 tol: float,
                     停止训练的最大误差值.
